require_relative 'ios_provisioning_profile_entitlements.rb'

module IOSProvisioningProfileParser
  # INFO: Based on https://github.com/fastlane/fastlane/blob/master/fastlane_core/lib/fastlane_core/provisioning_profile.rb
  # and http://maniak-dobrii.com/extracting-stuff-from-provisioning-profile/
  class ProvisioningProfile
    attr_reader :profile_path

    def initialize(path)
      @profile_path = path
      @profile_dict = ProvisioningProfile.parse(path)
    end

    def identities
      return @profile_dict["DeveloperCertificates"].map { |cert|
        encoded_identity = Base64.encode64(cert.string)
        ProvisioningProfile.decode_identity(encoded_identity)
      }
    end

    def teams
      return @profile_dict["TeamIdentifier"]
    end

    def name
      return @profile_dict["Name"]
    end

    def bundle_id
      return @profile_dict["Entitlements"]["application-identifier"]
    end

    def bundle_id_no_team
      return self.bundle_id().partition('.').last
    end

    def uuid
      return @profile_dict["UUID"]
    end

    def entitlements
      return ProvisioningProfileEntitlements.new(@profile_dict["Entitlements"])
    end

    private

    def self.parse(path, keychain_path = nil)
      require 'plist'

      plist = Plist.parse_xml(ProvisioningProfile.decode(path, keychain_path))
      if (plist || []).count > 5
        plist
      else
        raise RuntimeError, "Error parsing provisioning profile at path '#{path}'"
      end
    end

    def self.decode(path, keychain_path = nil)
      require 'tmpdir'
      Dir.mktmpdir('fastlane') do |dir|
        err = "#{dir}/cms.err"
        # we want to prevent the error output to mix up with the standard output because of
        # /dev/null: https://github.com/fastlane/fastlane/issues/6387
        if keychain_path.nil?
          decoded = `security cms -D -i "#{path}" 2> #{err}`
        else
          decoded = `security cms -D -i "#{path}" -k "#{keychain_path.shellescape}" 2> #{err}`
        end

        raise RuntimeError, "Failure to decode #{path}. Exit: #{$?.exitstatus}: #{File.read(err)}" if $?.exitstatus != 0

        decoded
      end
    end

    def self.decode_identity(encoded_identity)
      result = `echo \"#{encoded_identity}\" | base64 -D | openssl x509 -noout -inform DER -subject | sed 's/^.*CN=\\([^\\/]*\\)\\/.*$/\\1/'`.strip
      result
    end
  end
end